----------------------------------------------------------------------------------
-- 
-- Create Date:    17:19:40 08/07/2009 
-- Design Name:    
-- Module Name:    cpld - Behavioral 
-- Project Name:   MiniLA
-- Target Devices: XC9572(XL)
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity cpld is
 Port (
      clk_q:      in  std_logic;                        -- quartz clk
      clk_ext:    in  std_logic;                        -- external clk (uC Clk)
      adr_out:    out std_logic_vector(14 downto 0);    -- RAM address
      Ram_Read:   in  std_logic;                        -- active low
      Ram_We:     out std_logic;                        -- active low

      trig_out:   out std_logic;                        -- triggered (active high)
      Trig_en:    in  std_logic;                        -- trigger enable (active low)
      Trig_ext:   in  std_logic;                        -- external trigger

      Data_in:    in  std_logic_vector (7 downto 0);    -- data input

      sh_dat:     in  std_logic;                        -- shiftreg data
      sh_clk:     in  std_logic                         -- shiftreg clk
 ); 
end cpld;

-- config bit stream
-- bit-number  20-19        18-17    16-8   7-0
--             trigger_cfg  clk_mux  twert  mask

architecture Behavioral of cpld is

signal counter_q:    std_logic_vector(14 downto 0);
signal mask:         std_logic_vector (7 downto 0);  -- mask shift reg
signal twert:        std_logic_vector (7 downto 0);  -- twert shift reg

signal clk_div:      std_logic_vector (1 downto 0);  -- clk cfg shift reg
signal clk_mux:      std_logic_vector (1 downto 0);  -- clk select shift reg
signal clk_intern:   std_logic;                      -- divided RAM clk

-- trigger_cfg(1-0)
--0x: internal trigger 
--10: ext. trigger on low  level
--11: ext. trigger on high level
signal trigger_cfg:  std_logic_vector (1 downto 0);  -- trigger cfg shift reg

--- 'local variables' ----------------
signal rs_set:       std_logic;
signal rs_clr:       std_logic;
signal trig_out_q:   std_logic;
signal trigger:      std_logic;
signal trigger_ext:  std_logic;


begin

Ram_We <= not (clk_intern and Ram_Read);

--- clk divider and mux --------------
clk_divider: process (clk_q)
begin
  if clk_q'event and clk_q ='1' then
      clk_div <= clk_div +1;
  end if;
end process;

clk_intern <= clk_div(1) when (clk_mux = "01" and Trig_en = '0') else -- 20 MHz
              clk_div(0) when (clk_mux = "10" and Trig_en = '0') else -- 40 MHz
              clk_q      when (clk_mux = "11" and Trig_en = '0') else -- 80 MHz
              not clk_ext;                                            -- C

--- address counter for RAM access ---
counter: process (clk_intern)
begin
   if (clk_intern'event and clk_intern='0') then
     counter_q <= counter_q + 1;
   end if;  
end process;
adr_out <= counter_q;


--- Trigger logic --------------------
trigger <= '0' when (twert = (mask or Data_in)) else '1';
trigger_ext <= trig_ext xor trigger_cfg(0);

rs_set <= trigger when (trigger_cfg(1) = '0' ) else trigger_ext;
rs_clr <= Ram_read;

trig_latch: process(trigger, rs_set, rs_clr, Trig_en )
begin
  if (rs_clr = '0') then
     trig_out_q <= '0';
  elsif ( rs_set = '0' and Trig_en = '0') then
     trig_out_q <= '1';
  end if;  
end process;
trig_out <= trig_out_q;


--- Shift registers ------------------
shifter: process (sh_clk,sh_dat)
begin
   if (sh_clk'event and sh_clk='1') then
     mask        <= mask(6 downto 0) & sh_dat;
     twert       <= twert(6 downto 0) & mask(7);
     clk_mux     <= clk_mux(0) & twert(7);
     trigger_cfg <= trigger_cfg(0) & clk_mux(1);
   end if;
end process;  

end Behavioral;

